# -*-coding:utf-8-*-

# programmed by Y. Hwang (ETRI, Quantum Computing Research Lab.)

# version 0.6

from . import shortestpath

qubit_connectivity_graph = None

def initialize_graph(qubit_connectivity):
    """
        initialize graph for shortest path over nodes in graph
    """
    qubit_connectivity_graph = shortestpath.Graph()

    for i in qubit_connectivity.keys():
        qubit_connectivity_graph.add_vertex(i)

    for k, v in qubit_connectivity.items():
        for j in v:
            if k in qubit_connectivity[j]:
                qubit_connectivity_graph.add_edge(k, j, True)  # bi-directional
            else:
                qubit_connectivity_graph.add_edge(k, j, False)  # one-directional

    return qubit_connectivity_graph


def findQubitConnectionPath(qubit_connectivity, qubitA, qubitB):

    # initialize a graph adjacency matrix from the input relation
    qubit_connectivity_graph = initialize_graph(qubit_connectivity) 

    # find a path from the node i to the node j on the graph
    shortestpath.dijkstra(qubit_connectivity_graph, qubit_connectivity_graph.get_vertex(qubitA), qubit_connectivity_graph.get_vertex(qubitB))
    target = qubit_connectivity_graph.get_vertex(qubitB)

    path = [qubitB]
    shortestpath.shortest(target, path)
    
    path.reverse()
    temp_path_list = []

    for k in range(len(path)-1):
        qubit_a = qubit_connectivity_graph.get_vertex(path[k])
        qubit_b = qubit_connectivity_graph.get_vertex(path[k+1])

        temp_path_list.append((path[k], path[k+1], qubit_a.get_weight(qubit_b)))

    return temp_path_list

    
def findALLConnectionPath(qubit_connectivity):

    qubit_connection_path = {}
    for i in qubit_connectivity.keys():
        for j in qubit_connectivity.keys():

            # initialize a graph adjacency matrix from the input relation
            qubit_connectivity_graph = initialize_graph(qubit_connectivity)
            # find a path from the node i to the node j on the graph
            shortestpath.dijkstra(qubit_connectivity_graph, qubit_connectivity_graph.get_vertex(i), qubit_connectivity_graph.get_vertex(j))
            target = qubit_connectivity_graph.get_vertex(j)

            path = [j]
            shortestpath.shortest(target, path)

            path.reverse()
            temp_path_list = []

            for k in range(len(path)-1):
                qubit_a = qubit_connectivity_graph.get_vertex(path[k])
                qubit_b = qubit_connectivity_graph.get_vertex(path[k+1])

                temp_path_list.append((path[k], path[k+1], qubit_a.get_weight(qubit_b)))

            qubit_connection_path[(i, j)] = temp_path_list

    return qubit_connection_path


